﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

Static StrConstant k_fullpackageFolder = "root:Packages:imgT"
Static StrConstant k_pvfolder = "imgT_processvariables"
Static StrConstant k_fullpvfolder = "root:Packages:imgT:imgT_processvariables"
Static StrConstant k_fullpanel = "imgT_display#imgT_panel"
Static StrConstant k_imgDisplay = "imgT_display"

// uncomment to debug
//#define DEBUG

#ifndef DEBUG
#pragma hide=1
#endif

// ***
// STRUCTURES

// background settings
Static Structure S_BackgroundParams
	variable plorder
	variable onplane
	variable useimg
endStructure

Function Sf_GetBackgroundParams(sbg)
	STRUCT S_BackgroundParams &sbg
	
	ControlInfo/W=$k_fullpanel setvarPlOrder_tab1
	sbg.plorder = v_value
	ControlInfo/W=$k_fullpanel popupBckonStack_tab1
	sbg.onplane = v_value
	ControlInfo/W=$k_fullpanel checkAsBackground_tab1
	sbg.useimg = v_value
	
	return 0
end

// structure to store process variables
Structure S_CalibrateVariables
	// how: 1 pixel, 2 width, 3 height
	variable xyhow
	// panel inputs
	variable pivalue
	variable pilog
	// meters per pixel
	variable mpp
	// image xy size (meters)
	variable xyiwidth
	variable xyiheight
	// store as global
	variable asglobal
EndStructure

Function Sf_GetXYCalibrationInputs(scv)
	STRUCT S_CalibrateVariables &scv

	ControlInfo/W=$k_fullpanel popupmethodxy_tab1
	scv.xyhow = v_value
	ControlInfo/W=$k_fullpanel psizem_tab1
	scv.pivalue = v_value
	ControlInfo/W=$k_fullpanel psizelm_tab1
	scv.pilog = v_value
	ControlInfo/W=$k_fullpanel checkxyglobal_tab1
	scv.asglobal = v_value

	return 0
end

Function Sf_SaveCalibrateVariables(scv)
	STRUCT S_CalibrateVariables &scv

	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string iwf	
	
	DFREF cwd = GetDataFolderDFR()

	// always save local
	// (clumsy but future proof)
	iwf = "root:" + cwdfolder	
	SetDataFolder $iwf	
	NewDataFolder/O/S $k_pvfolder
	variable/G xyhow, mpp, xyiwidth, xyiheight, pivalue, pilog
	SetDataFolder cwd
	iwf += ":" + k_pvfolder
	NVAR/SDFR=$iwf xyhow, mpp, xyiwidth, xyiheight, pivalue, pilog
	xyhow = scv.xyhow
	mpp = scv.mpp
	xyiwidth = scv.xyiwidth
	xyiheight = scv.xyiheight
	pivalue = scv.pivalue
	pilog = scv.pilog
	
	if (scv.asglobal)
		// validate/create globals anew each time
		// (clumsy but future proof)
		SetDataFolder pf
		NewDataFolder/O/S $k_pvfolder
		variable/G xyhow, mpp, xyiwidth, xyiheight, pivalue, pilog
		SetDataFolder cwd
		iwf = k_fullpackageFolder + ":" + k_pvfolder
		NVAR/SDFR=$iwf xyhow, mpp, xyiwidth, xyiheight, pivalue, pilog
		xyhow = scv.xyhow
		mpp = scv.mpp
		xyiwidth = scv.xyiwidth
		xyiheight = scv.xyiheight
	endif
	
	return 0
end

// function to fix variable names
// lost translations in update from 1.x --> 2.x
// need: xyhow, pivalue, pilog, mpp, xyiwidth, xyiheight
// may have: xyhow, xympixel, xylmpixel, xyiwidth, xyliwidth
Function Sf_FixCalibrateVariables(wf)
	string wf
	
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder $wf
	if (exists("xympixel") == 2)
		NVAR xympixel, xylmpixel, xyiwidth, xyliwidth
		Variable/G mpp
		xyiwidth *= 10^xyliwidth
		Rename xympixel pivalue
		Rename xylmpixel pilog
		Rename xyliwidth xyiheight
		Variable/G xyiheight
	 	xyiheight = xyiwidth
	endif
	SetDataFolder cdf
	return 0
end

// get calibration variables
Function Sf_RecallCalibrateVariables(scv)
	STRUCT S_CalibrateVariables &scv

	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string iwf
	
	// check local
	iwf = "root:" + cwdfolder + ":" + k_pvfolder
	if (DataFolderExists(iwf))
		// fix for update that did not correctly change parameter names
		Sf_FixCalibrateVariables(iwf)		
		NVAR/SDFR=$iwf/Z xyhow, mpp, xyiwidth, xyiheight, pivalue, pilog
		scv.xyhow = xyhow
		scv.mpp = mpp
		scv.xyiwidth = xyiwidth
		scv.xyiheight = xyiheight
		scv.pivalue = pivalue
		scv.pilog = pilog
		return 0
	endif
	
	// check global
	if (DataFolderExists(k_fullpvfolder))
		iwf = k_fullpvfolder
		Sf_FixCalibrateVariables(iwf)		
		NVAR/SDFR=$iwf/Z xyhow, mpp, xyiwidth, xyiheight, pivalue, pilog
		scv.xyhow = xyhow
		scv.mpp = mpp
		scv.xyiwidth = xyiwidth
		scv.xyiheight = xyiheight
		scv.pivalue = pivalue
		scv.pilog = pilog
		return 1
	endif
	
	// neither local or global
	return -1
end

// ***
// HELPER FUNCTIONS

// draw tab1 - calibrate
Function disp_Tab1Draw(pwidth,pheight, ptabpos)
	variable pwidth, pheight, ptabpos

	variable tvoffset = ptabpos + 30
	
	TabControl tab_main, value=1, userdata(ptab)="1"
	
	TitleBox calibrate_tab1,pos={12,tvoffset},size={84,22},title="Calibrate XY"
	TitleBox calibrate_tab1,fSize=16,frame=0,fStyle=1,disable=1
	
	Button buttonclearlocalscale_tab1,pos={pwidth-280,tvoffset},size={76,16},title="Clear Local"
	Button buttonclearlocalscale_tab1,proc=X_ButtonProc,disable=1
	
	Button buttonclearglobalscale_tab1,pos={pwidth-180,tvoffset},size={76,16},title="Clear Global"
	Button buttonclearglobalscale_tab1,proc=X_ButtonProc,disable=1
	
	CheckBox checkxyglobal_tab1,pos={pwidth-95,tvoffset},size={76,16},title="save as global"
	CheckBox checkxyglobal_tab1,value=0, disable=1

	tvoffset += 25
	
	PopupMenu popupmethodxy_tab1,pos={20,tvoffset},size={113,23},title="How"
	PopupMenu popupmethodxy_tab1,mode=1,popvalue="pixel size",value= #"\"pixel size;image width;image height;\""
	PopupMenu popupmethodxy_tab1,disable=1,proc=X_PopMenuProc
	
	SetVariable psizem_tab1, pos={12+150,tvoffset},size={85,16},title="[m]",format="%1.3f"
	SetVariable psizem_tab1, value=_NUM:1, limits={0.1,10,0.1}, disable=1
	SetVariable psizelm_tab1, pos={12+240,tvoffset-3},size={70,16},title="\[010\Sn\M"
	SetVariable psizelm_tab1, value=_NUM:1, limits={-10,10,1}, disable=1
	
	Button buttonSetXY_tab1,pos={pwidth-85,tvoffset-5},size={75,25},proc=X_ButtonProc,title="Apply"
	Button buttonSetXY_tab1, disable=1
	
	// background
	
	tvoffset += 40
		
	TitleBox background_tab1,pos={12,tvoffset},size={84,22},title="Remove Background"
	TitleBox background_tab1,fSize=16,frame=0,fStyle=1,disable=1
//	
//	CheckBox checkAsBackground_tab1, pos={210,tvoffset+2},size={20,20}, proc=X_CheckProc
//	CheckBox checkAsBackground_tab1, title="use this image", value=0, disable=1
	
	PopupMenu popupBckonStack_tab1,pos={pwidth-110,tvoffset},size={100,23},title=""
	PopupMenu popupBckonStack_tab1,mode=1,value="This Layer;All Layers;",disable=1
//
//	CheckBox checkOTPlane_tab1,pos={pwidth-110,tvoffset},size={80,20},title="to this plane only"
//	CheckBox checkOTPlane_tab1,value=1, disable=1
	
	tvoffset += 25

	Button buttonsetbckgrnd_tab1,pos={20,tvoffset},size={96,20},proc=X_ButtonProc,title="Set Locations"
	Button buttonsetbckgrnd_tab1,userdata="start",disable=1

	Button buttoneditbckpoints_tab1,pos={120,tvoffset},size={40,20},proc=X_ButtonProc,title="Edit"
	Button buttoneditbckpoints_tab1,disable=1

	Button buttonclearbckpoints_tab1,pos={165,tvoffset},size={40,20},proc=X_ButtonProc,title="Clear"
	Button buttonclearbckpoints_tab1,disable=1
	
	SetVariable setvarplorder_tab1,pos={227,tvoffset},size={97,23},title="Plane Order"
	SetVariable setvarplorder_tab1,limits={1,4,1},value=_NUM:1, disable=1
//	
//	PopupMenu popupBckMask_tab1,pos={140,tvoffset},size={120,23},title="RoI"
//	PopupMenu popupBckMask_tab1,mode=1,value=f_BckgRoIs(),disable=1
	
	SetVariable setvarBckgImage_tab1,pos={20,tvoffset},size={280,23},title="Image: "
	SetVariable setvarBckgImage_tab1,value=_STR:"", disable=1

	Button buttonRemoveBckg_tab1,pos={pwidth-85,tvoffset-5},size={75,25},proc=X_ButtonProc,title="Apply"
	Button buttonRemoveBckg_tab1,disable=1

	return 0
end

// update tab 1
Function disp_updateTab1()

	variable hasBGM = f_HasBckgMrkrs()
	variable istack = f_IsStack()
	variable useImage = 0
	string usd

	//ControlInfo/W=$k_fullpanel popupmethodxy_tab1
	SetVariable setvarSize_tabM, win=$k_fullpanel, value=_STR:f_ImageSizeText()
	Button buttonclearlocalscale_tab1, win=$k_fullpanel, disable=(f_HasXYScale(1)==0)			
	Button buttonclearglobalscale_tab1, win=$k_fullpanel, disable=(f_HasXYScale(0)==0)			

	STRUCT S_CalibrateVariables scv
	Sf_RecallCalibrateVariables(scv)
	if (scv.xyhow!=0)
		PopupMenu popupmethodxy_tab1, win=$k_fullpanel, mode=scv.xyhow
		SetVariable psizem_tab1, win=$k_fullpanel, value=_NUM:scv.pivalue
		SetVariable psizelm_tab1, win=$k_fullpanel, value=_NUM:scv.pilog
	else
		SetVariable psizem_tab1, win=$k_fullpanel, value=_NUM:1
		SetVariable psizelm_tab1, win=$k_fullpanel, value=_NUM:0
	endif

	//ControlInfo/W=$k_fullpanel checkAsBackground_tab1
	//useImage = v_value
	useImage = 0
	if (useImage)
		SetVariable setvarplorder_tab1, win=$k_fullpanel, disable=1
		Button buttonsetbckgrnd_tab1,win=$k_fullpanel,disable=1
		Button buttoneditbckpoints_tab1,win=$k_fullpanel, disable=1
		Button buttonclearbckpoints_tab1, win=$k_fullpanel,disable=1
		Button buttonRemoveBckg_tab1, win=$k_fullpanel, disable=0
		SetVariable setvarBckgImage_tab1,win=$k_fullpanel, value=_STR:"Some Image", disable=2
		PopupMenu popupBckonStack_tab1, win=$k_fullpanel, disable=(!istack)
		//CheckBox checkAsBackground_tab1, win=$k_fullpanel, disable=(2*istack)
	else
		SetVariable setvarplorder_tab1, win=$k_fullpanel, disable=0
		Button buttonsetbckgrnd_tab1,win=$k_fullpanel,disable=0
		Button buttoneditbckpoints_tab1,win=$k_fullpanel, disable=(2*!hasBGM)
		Button buttonclearbckpoints_tab1, win=$k_fullpanel,disable=(2*!hasBGM)
		Button buttonRemoveBckg_tab1, win=$k_fullpanel, disable=(2*!hasBGM)
		SetVariable setvarBckgImage_tab1,win=$k_fullpanel, value=_STR:"", disable=1
		if (istack)
			PopupMenu popupBckonStack_tab1, win=$k_fullpanel, disable=(2*!hasBGM)
		else
			PopupMenu popupBckonStack_tab1, win=$k_fullpanel, disable=1
		endif
		//CheckBox checkAsBackground_tab1, win=$k_fullpanel, disable=(istack)
	endif
	
	return 0
end

// ***
// ACTIONS

// clear XY scale
Function T1_ClearXYScale(where)
	variable where
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string iwf
	switch(where)
		case 0:		// global
			iwf = k_fullpackagefolder + ":" + k_pvfolder
			break
		case 1:		// local
			iwf = "root:" + cwdfolder + ":" + k_pvfolder
			break
	endswitch
	
	KillDataFolder/Z $iwf

	return 0
end

// set xy method
Function T1_SetXYMethod(how)
	variable how
	
//	PopupMenu mrulera_tab1, win=$k_fullpanel, disable = 1
	switch(how)
		case 4:	// ruler length
			PopupMenu mrulera_tab1, win=$k_fullpanel, disable = 0
			break
	endswitch
	return 0
end

// set the xy calibration
Function T1_SetXY()
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string swf = "root:" + cwdfolder
	wave/SDFR=$swf cwave = $cwfile
	
	variable iwpx, iw, asglobal
	variable pwidth = DimSize(cwave,0)
	variable pheight = DimSize(cwave,1)

	STRUCT S_CalibrateVariables scv

	// get values from panel
	
	Sf_GetXYCalibrationInputs(scv)
	
	switch(scv.xyhow)
		case 1:	// pixel size
			scv.mpp = scv.pivalue*10^scv.pilog
			scv.xyiwidth = pwidth*scv.mpp
			scv.xyiheight = pheight*scv.mpp
			break
		case 2:	// image width
			scv.xyiwidth = scv.pivalue*10^scv.pilog
			scv.mpp = scv.xyiwidth/pwidth
			scv.xyiheight = pheight*scv.mpp
			break
		case 3:	// image height
			scv.xyiheight = scv.pivalue*10^scv.pilog
			scv.mpp = scv.xyiheight/pheight
			scv.xyiwidth = pwidth*scv.mpp
			break
	endswitch
	
	// save values
	Sf_SaveCalibrateVariables(scv)
	
	return 0
end

// set the location points for the background
Function T1_SetBackgroundPoints()
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string usD, rcwd, rcwdf
	variable whi, hasBGM = f_HasBckgMrkrs()
	
	usD = GetUserData(k_fullPanel,"buttonsetbckgrnd_tab1","")
	strswitch(usD)
		case "start":
			Button buttonsetbckgrnd_tab1,win=$k_fullpanel,userdata="end",title="End..."
			SetWindow $k_imgDisplay, userdata(draw)="active"
			SetDrawLayer/W=$k_imgDisplay ProgFront
			GraphWaveDraw/W=$k_imgDisplay/L/T/O imgT_bckgY,imgT_bckgX
			ModifyGraph/W=$k_imgDisplay mode=3,marker=8,msize=5,mrkThick=2
			break
		case "end":
			DoUpdate/W=$k_imgDisplay
			Button buttonsetbckgrnd_tab1,win=$k_fullpanel,userdata="start",title="Set Locations"
			Button buttoneditbckpoints_tab1,win=$k_fullpanel, disable=0
			Button buttonclearbckpoints_tab1,win=$k_fullpanel, disable=0
			Button buttonRemoveBckg_tab1, win=$k_fullpanel, disable=(2*!hasBGM)
			GraphWaveEdit/W=$k_imgDisplay/T=1 scalewy
			GraphNormal/W=$k_imgDisplay
			SetDrawLayer/W=$k_imgDisplay UserFront
			SetWindow $k_imgDisplay, userdata(draw)=""
			// put the data points in the working folder
			DFREF cdf = GetDataFolderDFR()	
			wave/SDFR=cdf cbx = imgT_bckgX
			wave/SDFR=cdf cby = imgT_bckgY
			
			rcwd = "root:" + cwdfolder
			SetDataFolder $rcwd
			rcwd += ":imgT_background"
			NewDataFolder/O $rcwd
			SetDataFolder cdf
						
			rcwdf = rcwd + ":imgT_bckgY"
			duplicate/O cby $rcwdf
			rcwdf = rcwd + ":imgT_bckgX"			
			duplicate/O cbx $rcwdf
			
			break
	endswitch

	return 0
end

// clear the location points for the background
Function T1_EditBackgroundPoints()
	
	//wave/SDFR=root imgT_bckgY, imgT_bckgX
	
	SetDrawLayer/W=$k_imgDisplay ProgFront
	GraphWaveEdit/W=$k_imgDisplay/T=1 imgT_bckgY
	Button buttonsetbckgrnd_tab1,win=$k_fullpanel,userdata="end",title="End..."
	Button buttoneditbckpoints_tab1,win=$k_fullpanel, disable=2
	Button buttonclearbckpoints_tab1,win=$k_fullpanel, disable=2
	Button buttonRemoveBckg_tab1, win=$k_fullpanel, disable=2
	SetWindow $k_imgDisplay, userdata(draw)="active"
	
	return 0
end

// clear the location points for the background
Function T1_ClearBackgroundPoints()
		
	RemoveFromGraph/W=$k_imgDisplay/Z imgT_bckgY
	killwaves/Z imgT_bckgY, imgT_bckgX
	Button buttoneditbckpoints_tab1,win=$k_fullpanel, disable=2
	Button buttonclearbckpoints_tab1,win=$k_fullpanel, disable=2
	
	return 0
end

// remove background
Function T1_RemoveBackground()
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string wdf = "root:" + cwdfolder
	string sfile, wfile
	variable cplane = f_PlaneNumber(), bd = f_ImageBitDepth()
	variable imax=(2^8)-1, sf, ic, vmin, vmax, fpmax = 2^64 - 1
	variable bckgexists=0, nimgs, imin=0, po, onplane, useimg
	
	// avoid background removal on a background
	// set existence of background
	if (f_isBackground())
		bckgexists = 1
		sfile = ReplaceString("_bg",cwfile,"")
		wave/SDFR=$wdf simg = $sfile
		wfile = sfile + "_bg"
	else
		wave/SDFR=$wdf simg = $cwfile
		wfile = cwfile + "_bg"
		sfile = cwfile
	endif

	STRUCT S_BackgroundParams sbg
	Sf_GetBackgroundParams(sbg)
	
	po = sbg.plorder
	onplane = sbg.onplane
	useimg = sbg.useimg

	// set up for background removal
	wave/SDFR=$wdf/Z bimg = $wfile
	bckgexists = waveexists(bimg)
	
	// remove the background
	
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder $wdf
	
	// create new background
	if (bckgexists==0)
		duplicate simg $wfile
		wave wbimg = $wfile
		wbimg = 0
	endif
	
	// extract wave to create background
	if (f_isStack())
		nimgs = f_NImagesInStack()
	else
		onplane = 0
	endif
	
	// create the RoI mask
	wave RoIMask = GenerateBckgMask(simg)
	
	// wave declarations
	wave/SDFR=$wdf wbimg = $wfile

	// remove background
	switch(onplane)
		case 0:		// not a stack
			vmax = wavemax(simg)
			vmin = wavemin(simg)
			sf = (vmax - vmin)
			MatrixOP/FREE fbimg = fp64((simg - vmin)/sf)
			//Redimension/D fbimg			
			ImageRemoveBackground/P=(po)/O/R=RoIMask fbimg
			vmax = wavemax(fbimg)
			vmin = wavemin(fbimg)
			sf = imax/(vmax - vmin)
			switch(bd)
				case 8:
					MatrixOP/O fbimg = uint8(sf*(fbimg - vmin))
					break
				case 16:
					MatrixOP/O fbimg = uint16(sf*(fbimg - vmin))
					break
				case 64:
					MatrixOP/O fbimg = fp64(sf*(fbimg - vmin))
					break
			endswitch
			wbimg = fbimg
			break
		case 1:		// this plane only
			MatrixOP/FREE fbimg = simg[][][cplane]
			vmax = wavemax(fbimg)
			vmin = wavemin(fbimg)
			sf = (vmax - vmin)
			Redimension/D/N=(-1,-1,0) fbimg
			fbimg = (fbimg - vmin)/sf
			ImageRemoveBackground/O/P=(po)/R=RoIMask fbimg
			vmin = wavemin(fbimg)
			vmax = wavemax(fbimg)
			sf = imax/(vmax - vmin)
			switch(bd)
				case 8:
					MatrixOP/O fbimg = uint8(sf*(fbimg - vmin))
					break
				case 16:
					MatrixOP/O fbimg = uint16(sf*(fbimg - vmin))
					break
				case 64:
					MatrixOP/O fbimg = fp64(sf*(fbimg - vmin))
					break
			endswitch
			wbimg[][][cplane] = fbimg[p][q]
			//ImageTransform/D=wbimg/P=(cplane) setPlane, fbimg
			break
		case 2:		// all planes
			Duplicate/O simg, fbimgSTACK
			Redimension/D fbimgSTACK	
			for (ic=0;ic<nimgs;ic+=1)
				MatrixOP/O fbimg = layer(fbimgSTACK,ic)
				ImageRemoveBackground/O/P=(po)/R=RoIMask fbimg
				vmin = WaveMin(fbimg)
				vmax = WaveMax(fbimg)
				sf = imax/(vmax - vmin)
				fbimgSTACK[][][ic] = fbimg[p][q]
				//ImageTransform/P=(ic)/D=fbimgSTACK/O setPlane, fbimg
			endfor
//			Make/N=(nimgs)/FREE tmpWave
//			Duplicate/FREE simg, fbimgSTACK
//			Redimension/D fbimgSTACK    
//			MultiThread tmpWave[] = TS_RemoveBackground(fbimgSTACK,ROIMask,po,p)
//			MatrixOP/O wbimg = uint8(fbimgSTACK)
			switch(bd)
				case 8:
					MatrixOP/O wbimg = uint8(sf*(fbimg - vmin))
					break
				case 16:
					MatrixOP/O wbimg = uint16(sf*(fbimg - vmin))
					break
				case 64:
					MatrixOP/O wbimg = fp64(sf*(fbimg - vmin))
					break
			endswitch
			break
	endswitch
	
	killwaves/Z fbimg, RoIMask, fbimgSTACK
	
	SetDataFolder cdf

	return 0
end

ThreadSafe Function TS_RemoveBackground(WAVE fbimgSTACK, WAVE RoIMask, variable po, variable ic)

    variable sf
    MatrixOP/FREE fbimg = layer(fbimgSTACK,ic)
    ImageRemoveBackground/O/W/P=(po)/R=RoIMask fbimg
    WaveStats/Q/M=1 fbimg
    sf = 255/(V_max - V_min)
    MatrixOP/O fbimg = (sf*(fbimg - V_min))
    fbimgSTACK[][][ic] = fbimg[p][q]

    return 0

end

// generate an RoI mask from wimg
// how: 1 mask inside; 2 mask outside
// which: 0 histogram, 1 background, 2 threshold
Function/WAVE GenerateBckgMask(wimg)
	wave wimg
	
	variable npts, ic

	duplicate/O wimg mwimg
	wave imgMask = mwimg
	if (WaveDims(wimg) == 3)
		Redimension/B/U/N=(-1,-1,1) imgMask
	else
		Redimension/B/U imgMask
	endif
	
	wave/SDFR=root: imgT_bckgY, imgT_bckgX
	npts = numpnts(imgT_bckgY)
	imgMask = 64
	for (ic=0;ic<npts;ic+=1)
		imgMask[x2pnt(imgMask,imgT_bckgX[ic])][x2pnt(imgMask,imgT_bckgY[ic])] = 1
	endfor
	
	return mwimg
end

Function T1_ShowBackground()

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwfile
	
	cwfile += "_bg"
	
	disp_ShowAnImage()

	return 0
end