﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3

// v 2.40

#pragma version = 2.40
#pragma IgorVersion = 8

//#pragma ModuleName=ImageTools

#include "Image Tools Controls"
#include "Image Tools Load"
#include "Image Tools Calibrate"
#include "Image Tools Process"
#include "Image Tools Print"

Static StrConstant k_strversion = "2.40"
Static Constant k_version = 2.40

// uncomment to debug
//#define DEBUG

#ifndef DEBUG
#pragma hide=1
#endif

// ***
// VARIABLES

// package definitions
// folders
Static StrConstant k_packageFolder = "imgT"
Static StrConstant k_histfolder = "imgT_hist"
Static StrConstant k_cpfolder = "imgT_cprint"
Static StrConstant k_fullpackageFolder = "root:Packages:imgT"
Static StrConstant k_pvfolder = "imgT_processvariables"

// windows
Static StrConstant k_imgDisplay = "imgT_display"
//Static StrConstant k_histPanel = "Histograms"
Static StrConstant k_histGraphs = "imgT_histograms"
Static StrConstant k_panelName = "imgT_panel"
//Static StrConstant k_fullhpanel = "imgT_display#Histograms"
//Static StrConstant k_fullhgraphs = "imgT_display#Histograms#imgT_histograms"
Static StrConstant k_fullpanel = "imgT_display#imgT_panel"
Static StrConstant k_mqprofile = "MQprofile"
Static StrConstant k_fullmqprofile = "imgT_display#imgT_panel#MQprofile"

// panel and graph dimensions
Static Constant wleft = 30
Static Constant wtop = 50
Static Constant pwidth = 415
Static Constant pheight = 530
Static Constant ptabpos = 330
Static Constant gwidth = 640
Static Constant cbheight = 50
Static Constant k_minscreenh = 600

// menu placement
Menu "Packages"
	SubMenu "Image Processing"
		"Image Tools", /Q, init_StartImageTools()
	end
end

// ***
// Updater Package

Static Constant kCHECK_FOR_UPDATES=1  // Needed. Set to 1 to check for updates on init.
Static StrConstant ksSHORTNAME="Image Tools" // The project shortname on IgorExchange, needed if not the same as moduleName.
static StrConstant ksLOCATION="https://www.wavemetrics.com/project-releases/21002"

static function CheckForUpdates()
    if(kCHECK_FOR_UPDATES)
        #if (exists("Updater#UpdateCheck")==6)
            return Updater#UpdateCheck("fileloc:"+FunctionPath(""))
        #endif
    endif
    return 0
end

// ***
// UPDATER HOOK FUNCTION

// bla
// after compiling
Function AfterCompiledHook()

	if (CheckForUpdates())
		return 0 // this file has been updated; quit execution to allow user to reload
	endif

	if (!DataFolderExists(k_fullPackageFolder))
		init_InitializePackage()
	else
		imgT_Update()
	endif
	
	return 1
end

// updates occur within the imgT package folder
Function imgT_Update()
	string svn
	
	DFREF fpf = $k_fullPackageFolder
	NVAR/Z/SDFR=fpf k_imgTVersion

	string astr, estr
	astr = ""
	estr = "\r\rThe image window and panel will close and reopen.\rSettings may change, but data will not change."
	variable how = 0
	
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder fpf
	if (!NVAR_Exists(k_imgTVersion))
		astr = "Updating current undefined version of \rImage Tools to version " + k_strversion + ".\r\r"
		how = 1
	else
		if (k_imgTVersion != k_version)
			astr = "Updating old version of Image Tools " + num2str(k_imgTVersion) + " to newer version " + k_strversion + "."
			how = 1
		endif
	endif
	
	if (how == 1)		
		print date() + " " + Secs2Time(DateTime,3) + " " + astr
		astr += estr
		DoAlert/T="Updating Image Tools", 0, astr
		DoWindow/K imgT_display
		DoWindow/K k_histGraphs
		
		// update block
		strswitch(k_strversion)
			case "2.40":
			case "2.30":
			case "2.22":
			case "2.21":
			case "2.20":
			case "2.11":
			case "2.10":
				variable/G k_imgTVersion
				make/N=1/O scalewx, scalewy, scaledx
				k_imgTVersion = k_version
				break
		endswitch
		
		NVAR/SDFR=fpf k_imgTVersion
		k_imgTVersion = k_version
	endif
	
	SetDataFolder cdf
	
	print date() + " " + Secs2Time(DateTime,3) + " - Restarting Image Tools version " + k_strversion
	init_CreateGUIs(0)
	
	return 0
end

// ***
// STRUCTURES

// structure to collect folder + file values
Structure S_iDisplay
	string folder
	string file
EndStructure

// fill the folder + file structure from panel
Function Sf_GetImageDir(iD)
	STRUCT S_iDisplay &iD
	
	ControlInfo/W=$k_fullpanel popupfolder_tabM
	id.folder = "imgT_" + s_value
	ControlInfo/W=$k_fullpanel popupfile_tabM
	id.file = "img_" + s_value
	
	return 0
end

// structure to collect marquee or hRoI coordinates
Structure S_MarqueeCoordinates
	variable hasmarquee
	variable hasdrawing
	variable paleft
	variable paright
	variable patop
	variable pabottom
	variable pgleft
	variable pgright
	variable pgtop
	variable pgbottom
	variable imgmag
EndStructure

// fill the structure from drawing or image marquee
Function Sf_GetMQCoordinates(mc,[present])
	Struct S_MarqueeCoordinates &mc
	variable present
	
	variable pntsh, pxlsh, pntsppxl

	GetMarquee/W=$k_imgDisplay/Z
	mc.hasmarquee = v_flag
//	DrawAction/W=$k_imgDisplay/L=ProgFront getgroup=hRoI, extractOutline
//	mc.hasdrawing = v_flag
	
	if (!ParamIsDefault(present))
		return (mc.hasmarquee)
	endif
	
	mc.paleft = NaN; mc.patop = NaN; mc.paright = NaN; mc.pabottom = NaN
	mc.pgleft = NaN; mc.pgtop = NaN; mc.pgright = NaN; mc.pgbottom = NaN
	
	GetWindow $k_imgDisplay expand
	mc.imgmag = v_value
	
	if (mc.hasmarquee == 1)
		mc.paleft = (v_left)
		mc.paright = (v_right)
		mc.patop = (v_top)
		mc.pabottom = (v_bottom)
		GetMarquee/W=$k_imgDisplay left,top
		mc.pgleft = v_left
		mc.pgright = v_right
		mc.pgtop = v_top
		mc.pgbottom = v_bottom
#ifDEF DEBUG
		print "MQ from MARQUEE"
		print mc
#endif
	endif

	return mc.hasmarquee
end

// fill the structure from drawing
Function Sf_GetHRoICoordinates(mc,[present])
	Struct S_MarqueeCoordinates &mc
	variable present
	
	variable pntsh, pxlsh, pntsppxl

	DrawAction/W=$k_imgDisplay/L=ProgFront getgroup=hRoI, extractOutline
	mc.hasdrawing = v_flag
	
	if (!ParamIsDefault(present))
		return (mc.hasdrawing)
	endif
	
	mc.paleft = NaN; mc.patop = NaN; mc.paright = NaN; mc.pabottom = NaN
	mc.pgleft = NaN; mc.pgtop = NaN; mc.pgright = NaN; mc.pgbottom = NaN

	// has a drawing
	if (mc.hasdrawing == 1)
		wave w_polyx, w_polyy
		mc.pgleft=w_polyx[0]
		mc.pgright=w_polyx[1]
		mc.pgtop=w_polyy[1]
		mc.pgbottom=w_polyy[2]
		getwindow $k_imgDisplay, psizeDC
		pxlsh = V_right-V_left
		getwindow $k_imgDisplay, psize
		pntsh = V_right-V_left  
		pntsppxl = pntsh / pxlsh
		mc.paleft = PixelFromAxisVal(k_imgDisplay,"left",mc.pgleft)*pntsppxl
		mc.patop = PixelFromAxisVal(k_imgDisplay,"top",mc.pgtop)*pntsppxl + cbheight
		mc.paright = PixelFromAxisVal(k_imgDisplay,"left",mc.pgright)*pntsppxl
		mc.pabottom = PixelFromAxisVal(k_imgDisplay,"top",mc.pgbottom)*pntsppxl + cbheight
		killwaves/Z w_polyx, w_polyy
#ifDEF DEBUG
		print "MQ from DRAWING"
		print mc
#endif
	endif

	return mc.hasdrawing
end

// fill the folder + file structure manually
// pass paramters with prefixes!!!
Function SetImagetoShow(sfolder, sfile)
	string sfolder, sfile
	
	// globals for package	
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string flist
	variable itn
	
	// set the folder
	cwdfolder = sfolder	
	flist = f_LoimgT()
	itn = WhichListItem(sfolder[5,inf],flist) + 1	
	Popupmenu popupfolder_tabM, win=$k_fullpanel, mode=itn, value=f_LoImgT()
	
	// set the file
	cwfile = sfile
	flist = f_LoimgF()
	itn = WhichListItem(sfile[4,inf],flist) + 1
	Popupmenu popupfile_tabM, win=$k_fullpanel, mode=itn, value=f_LoimgF()
	
	return 0
end

// ***
// HOOK FUNCTIONS
//

// image window hook function
Function Hf_imgTImageHook(iwH)		// WindowHookFunction
	STRUCT WMWinHookStruct &iwH
	
	variable hookResult = 0, ctab
	string wmode
	
	switch(iwH.eventCode)
		case 0:		// active
			wmode = GetUserData(k_imgDisplay,"","draw")
			// drawing mode active
			if (strlen(wmode) != 0)
#ifdef DEBUG
				GetWindow $k_imgDisplay drawlayer
				print "drawing mode active in active " + s_value
#endif
				break
			endif
			hookResult = 1
			break
		case 2:		// graph being killed
			KillWindow/Z $k_histGraphs
			hookResult = 1
			break
		case 5:		// mouseup
#ifdef DEBUG
			print "mouseup on image"
#endif
			wmode = GetUserData(k_imgDisplay,"","draw")
			// drawing mode active
			if (strlen(wmode) != 0)
#ifdef DEBUG
				GetWindow $k_imgDisplay drawlayer
				print "drawing mode active " + s_value
#endif
				hookResult = 1
				break
			endif
			if (f_HasMQ())
				//disp_UpdateMQCsrValues()
				ResetMQCoordinates(force=1)
			endif
			ControlInfo/W=$k_fullpanel tab_main
			ctab = v_value
			disp_UpdatePanelControls(-1)
			disp_UpdatePanelControls(ctab)
			hookResult = 1
			break
	endswitch
	
	return hookResult
end

// histogram window hook function
Function Hf_imgTHistogramsHook(igH)		// WindowHookFunction
	STRUCT WMWinHookStruct &igH
	
	variable hookResult = 0, cx, cy
	string dtrace

	switch(igH.eventCode)
		case 7:		// cursor moved
#ifdef DEBUG
			print "cursor moved on histograms"
#endif
			GetWindow/Z $k_panelName exterior
			if (v_value==1)
#ifdef DEBUG
				print "setting cursor display values on panel: ", xcsr(A,k_histGraphs)
#endif
			// first time putting up the cursor
				ValDisplay valdispcx_tabM, win=$k_panelName, value=_NUM:xcsr(A,k_histGraphs)
				ValDisplay valdispcy_tabM, win=$k_panelName, value=_NUM:vcsr(A,k_histGraphs)
			endif
			hookresult = 1
			break
	endswitch
	
	return hookResult
end

// panel hook function
Function Hf_imgTPanelHook(ipH)		// WindowHookFunction
	STRUCT WMWinHookStruct &ipH
	
	variable hookResult = 0, il, ctab, pA, pB
	
	switch(ipH.eventCode)
		case 0:		// panel update
			ControlInfo/W=$k_fullpanel tab_main
			ctab = v_value
#ifdef DEBUG
			print "active at tab", ctab
#endif
			disp_UpdatePanelControls(-1)
			disp_UpdatePanelControls(ctab)
			hookResult = 1
			break
		case 2:		// panel being killed
			KillWindow/Z $k_histGraphs
			KillWindow/Z $k_imgDisplay
			hookResult = 1
			break
	endswitch
	
	return hookResult	
end

// switch tabs
Function disp_SwitchTab(ptab, ctab)
	string ptab
	variable ctab
	
#ifdef DEBUG
	print "Switching from tab to tab ", ptab, ctab
#endif
	
	string tabList, ctabstr=num2str(ctab)
	
	// hide prior controls
	ptab = "*_tab"+ptab
	tabList = ControlNameList(k_panelName,";",ptab)
	ModifyControlList tabList win=$k_fullpanel, disable=1
	
	// show new controls
	ptab = "*_tab"+ctabstr
	tabList = ControlNameList(k_panelName,";",ptab)
	ModifyControlList tabList win=$k_fullpanel, disable=0

	// set current tab value
	TabControl tab_main,win=$k_panelName,userdata(ptab)=ctabstr
	
	// clean up controls in special cases
	disp_UpdatePanelControls(ctab)		
	
	ControlUpdate/A/W=$k_panelName

	return 0
end

// change bit depth
Function ChangeImageBitDepth(minmax,fbd)
	variable minmax, fbd

	// globals for package	
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	variable bd = f_ImageBitDepth(), pntset
	
	switch(bd)
		case 8:
		case 16:
			bd = 2^bd - 1
			pntset = fbd*bd
			switch(minmax)
				case -2:		// monochrome onoff
					ModifyImage/W=$k_imgDisplay $cwfile, explicit=fbd
					break
				case -1:		// change min
					ModifyImage/W=$k_imgDisplay $cwfile, ctab={pntset,,,0}
					break
				case 0:		// reverse
					ModifyImage/W=$k_imgDisplay $cwfile, ctab={,,,fbd}
					break
				case 1:		// change max
					ModifyImage/W=$k_imgDisplay $cwfile, ctab={,pntset,,0}
					break
			endswitch
			break
		default:
			bd = f_DisplayedImageMax()
			pntset = fbd*bd
			switch(minmax)
				case -1:		// change min
					ModifyImage/W=$k_imgDisplay $cwfile, ctab={pntset,,,0}
					break
				case 0:		// reverse
					ModifyImage/W=$k_imgDisplay $cwfile, ctab={,,,fbd}
					break
				case 1:		// change max
					ModifyImage/W=$k_imgDisplay $cwfile, ctab={,pntset,,0}
					break
			endswitch
			break
	endswitch
	
	return pntset
end

// ***
// STARTUP

// check screen height
Function init_CheckScreen()

	variable rtn = 0, nitems

	variable first = 0, sheight
	string sinfo
	
	sinfo = StringbyKEY("SCREEN1",IgorInfo(0))
	nitems = ItemsInList(sinfo,",")
	sinfo = StringFromList(nitems-1,sinfo,",")
	sheight = str2num(sinfo)
	
	if (sheight < k_minscreenh)
		rtn = -1
	endif
	
	return rtn
end

// starting point from menu call
Function init_StartImageTools()

	variable first = 0
	
	// initialize?	
	if (!DataFolderExists(k_fullpackageFolder))
		init_InitializePackage()
		first = 1
	else
		imgT_Update()
	endif

	init_CreateGUIs(first)
	return 0
end

Function init_CreateGUIs(first)
	variable first

	string scrwarning = "This package can only be used on a screen with a minimum height of " + num2str(k_minscreenh) + " pixels. Change your screen resolution and try again."
	
	if (init_CheckScreen() < 0)
		DoAlert/T="Screen Too Small" 0, scrwarning
		return 0
	endif
	
	string lwfile
	// bring window forward or create?
	DoWindow/F $k_imgDisplay
	if (v_flag == 0)
		// validate working folder and file
		DFREF pf = $k_fullpackageFolder
		SVAR/SDFR=pf cwdfolder, cwfile
		if (!first)
			lwfile = "root:" + cwdfolder + ":" + cwfile
			wave swave = $lwfile
			if (!waveExists(swave))
				cwdfolder = StringFromList(0,f_LoimgT())
				cwfile = "img_" + StringFromList(0,f_LoimgF(sfolder=cwdfolder))
			endif
		endif
		// create new windows
		disp_CreateImageWindow(0)
		disp_ShowHistograms(0)
		disp_AppendImageToolsPanel(0)
		// initialize the tabs
		if (f_NImagesLoaded() == 0)
			disp_InitializeTabMenus(-1)
		else
			disp_InitializeTabMenus(100)
			disp_ResetImageFileonPanel()
			T3_UpdateContactPrint()
		endif
		disp_SetHookFunctions()
	endif
		
	return 0
end

// set all window hook functions
Function disp_SetHookFunctions()

	SetWindow $k_imgDisplay, hook(imgTImageHook)=Hf_imgTImageHook
	SetWindow $k_histGraphs, hook(imgTHistHook)=Hf_imgTHistogramsHook
	SetWindow $k_panelName, hook(imgTPanelHook)=Hf_imgTPanelHook
	//SetWindow $k_fullmqprofile hook(MQProfile)=Hf_mqProfileHook

	return 0
end

// initialize the package folder
Static Function init_InitializePackage()

	// create package folder
	SetDataFolder root:	
	NewDataFolder/O/S Packages	
	NewDataFolder/S $k_packageFolder
	
	// create globals
	// working directory
	string/G cwdfolder = "Packages:imgT", cwfile = "img_default"
	make/N=1/O scalewx, scalewy, scaledx
	make/N=101/D/O mqprofile = NaN
	make/N=(600,400)/B/U img_default
	img_default = 200*sin(x)*cos(y)+200
	variable/G k_imgTVersion = k_version
	NewDataFolder/O contactprint
	
	SetDataFolder root:

	return 0
end

// create image window
Function disp_CreateImageWindow(LoR)
	variable LoR

	// globals for package	
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	wave/SDFR=pf/Z scalewx, scalewy, scaledx
	
	string stitle, fstr, tstr
	variable lpos = wleft+LoR*pwidth
	
	// set the folder and image
	fstr = "root:" + cwdfolder
	DFREF imgF = $fstr	
	wave/SDFR=imgF itd = $cwfile

	variable bd 

	NewImage/N=$k_imgDisplay itd
	MoveWindow/W=$k_imgDisplay lpos,wtop,-1,-1			
	ModifyImage/W=$k_imgDisplay $cwfile, explicit=0
	ModifyGraph/W=$k_imgDisplay height={Plan,1,left,top}
	
	// set grayscale range
	switch(bd)
		case 8:
		case 16:
			bd = (2^f_ImageBitDepth()) - 1
			ModifyImage/W=$k_imgDisplay $cwfile, ctab={0,bd,Grays,0}
			break
		default:
			ModifyImage/W=$k_imgDisplay $cwfile, ctab={0,*,Grays,0}
			break
	endswitch
	
	// add scale bar (initially in hidden state)
	AppendToGraph/W=$k_imgDisplay/T scalewy vs scalewx
	ErrorBars/L=3 scalewy X,wave=(scaledx,scaledx)
	ModifyGraph/W=$k_imgDisplay/Z hidetrace(scalewy)=1, mode(scalewy)=2, rgb(scalewy)=(65535,65534,49151)
	
	disp_AddControlBar()
	
	return 0
end

// add control bar
Function disp_AddControlBar()

	variable nimgs = f_NImagesLoaded()
	
	ControlBar cbheight
	
	DefaultGuiFont/W=# all={"_IgorSmall",10,0}, button={"_IgorMedium",12,0}
	
	PopupMenu popupSplit_CB, pos={5,2},size={80,20},title="Split"
	PopupMenu popupSplit_CB, mode=1, value=f_SplitOptions(), proc=X_PopMenuProc
	
	PopupMenu popupColor_CB, pos={5,25},size={80,20},title=""
	PopupMenu popupColor_CB, mode=1, value=CTabList(), proc=X_PopMenuProc
	
	Slider sliderIMin_CB, pos={185,1},size={100,20},vert=0,limits={0,0.5,0.02},value=0,proc=X_SliderProc

	Button buttonResetIRange_CB, pos={290,10},size={40,22},title="<-->",proc=X_ButtonProc

	Slider sliderIMax_CB, pos={335,1},size={100,20},vert=0,limits={0.5,1,0.02},value=1,proc=X_SliderProc

	CheckBox checkInvert_CB,pos={435,5},size={20,20},title="Invert",value=0,proc=X_CheckProc

	CheckBox checkBW_CB,pos={435,25},size={20,20},title="B&W",value=0,proc=X_CheckProc

	return 0
end

// move histogram cursor
Function disp_MoveHistCursor(which,pntset)
	variable which, pntset
	
	string dtrace
	variable xend
	
	switch(which)
		case -1:		// left
			dtrace = CsrWave(C,k_histgraphs,1)
			Cursor/P/W=$k_histgraphs C $dtrace, pntset
			break
		case 0:		// both
			wave dwave = CsrWaveRef(C,k_histGraphs)
			xend = numpnts(dwave) - 1
			dtrace = CsrWave(C,k_histgraphs,1)
			Cursor/P/W=$k_histgraphs C $dtrace, 0
			Cursor/P/W=$k_histgraphs D $dtrace, xend
		case 1:		// right
			dtrace = CsrWave(D,k_histgraphs,1)
			Cursor/P/W=$k_histgraphs D $dtrace, pntset
			break
	endswitch
	
	return 0
end

// show histograms
// how = 0 new, 1 replace traces
Function disp_ShowHistograms(how)
	variable how

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string workf = "root:" + cwdfolder
	DFREF wf = $workf

	variable dhmin, dhmax, xmid, xend
	string imgroot, iHwave, iCHwave, iDHwave, dhistlimits
	string alltraces, htracen, htracel, ctrace, dtrace

	// regenerate the histogram each time this is called
	ControlInfo/W=$k_fullpanel popupHistRoI_tabM
	if (v_flag == 0)
		GenerateHistograms(0)
	else	
		GenerateHistograms(v_value - 1)
	endif
	
	workf += ":" + k_histFolder
	DFREF hf = $workf
	
	imgroot = StringFromList(1,cwfile,"_")
	iHwave = "hist_" + imgroot
	iCHwave = "chist_" + imgroot
	iDHwave = "dhist_" + imgroot
	
	wave/SDFR=hf theH = $iHwave
	wave/SDFR=hf theCH = $iCHwave
	wave/SDFR=hf theDH = $iDHwave
	
	WaveStats/Q theDH
	dhmax = v_max
	dhmin = v_min
	sprintf dhistlimits, "%d,%d",dhmin,dhmax
	
	// how = 0 is new; how = 1 is replace
	switch(how)
		case 0:		// new
			//DoWindow/F $k_imgDisplay
//#ifndef DEBUG
//			NewPanel/EXT=2/HOST=$k_imgDisplay/N=$k_histPanel/W=(0,0,(gwidth+1),0.25*gwidth)/K=2
//			ModifyPanel noEdit=1, fixedSize=1
//#else
//			NewPanel/EXT=2/HOST=$k_imgDisplay/N=$k_histPanel/W=(0,0,(gwidth+1),0.25*gwidth)
//#endif
			KillWindow/Z $k_histGraphs
			Display/N=$k_histGraphs/W=(wleft,wtop+pheight,gwidth,wtop+pheight+0.25*gwidth)/K=2
			ModifyGraph/W=$k_histGraphs height={Aspect,0.25} //, height=0.25*gwidth
			AppendtoGraph/W=$k_histGraphs/R=lhist theH
			htracel = StringFromList(0,TraceNameList("",";",1))
			AppendtoGraph/W=$k_histGraphs/L=nhist theH
			htracen = StringFromList(1,TraceNameList("",";",1))
			AppendToGraph/W=$k_histGraphs/L=chist theCH
			ctrace = StringFromList(2,TraceNameList("",";",1))
			AppendtoGraph/W=$k_histGraphs/R=dhist theDH
			dtrace = StringFromList(3,TraceNameList("",";",1))
			xend = numpnts(theDH) - 1
			xmid = round(numpnts(theDH)/2)
			
			ModifyGraph/W=$k_histGraphs rgb=(52428,1,1),lsize=2,standoff=0
			ModifyGraph/W=$k_histGraphs rgb($htracen)=(1,4,52428), rgb($ctrace)=(1,4,52428)
			ModifyGraph/W=$k_histGraphs log(lhist)=1,freePos(chist)={0,bottom},freePos(dhist)={inf,bottom}
			ModifyGraph/W=$k_histGraphs noLabel(lhist)=2,noLabel(lhist)=2,nticks(nhist)=0,nticks(nhist)=0,tick(lhist)=3
			ModifyGraph/W=$k_histGraphs noLabel(chist)=2,noLabel(dhist)=2,nticks(chist)=0,nticks(dhist)=0,zero(dhist)=2
			ModifyGraph/W=$k_histGraphs mode($htracen)=1,mode($htracel)=1,lsize($htracen)=1,lsize($htracel)=1
			ModifyGraph/W=$k_histGraphs freePos(lhist)={inf,bottom},freePos(nhist)={0,bottom}
			ModifyGraph/W=$k_histGraphs axisEnab(nhist)={0,0.5},axisEnab(lhist)={0,0.5},noLabel(nhist)=2
			ModifyGraph/W=$k_histGraphs axisEnab(chist)={0.51,1},axisEnab(dhist)={0.51,1}
			ModifyGraph/W=$k_histGraphs tick(bottom)=1,noLabel(bottom)=2
			Cursor/W=$k_histGraphs A $dtrace, xmid
			Cursor/W=$k_histGraphs/P/S=2/L=1/H=2/N=1/A=0 C $dtrace, 0
			Cursor/W=$k_histGraphs/P/S=2/L=1/H=2/N=1/A=0 D $dtrace, xend
			ScaleDHistogram(1)
			break
		case 1:		// just replace waves
			alltraces = SortList(TraceNameList(k_histGraphs,";",1))
			// alphabetical sort: c, d, h
			ctrace = StringFromList(0,alltraces)
			ReplaceWave/W=$k_histGraphs trace=$ctrace, theCH
			dtrace = StringFromList(1,alltraces)
			ReplaceWave/W=$k_histGraphs trace=$dtrace, theDH
			htracel = StringFromList(3,alltraces)
			ReplaceWave/W=$k_histGraphs trace=$htracel, theH
			htracen = StringFromList(2,alltraces)
			ReplaceWave/W=$k_histGraphs trace=$htracen, theH
			//DoUpdate
			if (strlen(CsrInfo(A,k_histGraphs))==0)
				ValDisplay valdispcx_tabM, win=$k_fullpanel, value=_NUM:NaN
				ValDisplay valdispcy_tabM, win=$k_fullpanel, value=_NUM:NaN
			else
				ValDisplay valdispcx_tabM, win=$k_fullpanel, value=_NUM:xcsr(A,k_histGraphs)
				ValDisplay valdispcy_tabM, win=$k_fullpanel, value=_NUM:vcsr(A,k_histGraphs)
			endif
			// scale the histogram according to slider
			ControlInfo/W=$k_fullpanel slidermagdhist_tabM
			ScaleDHistogram(v_value)
			break
	endswitch
	SetWindow $k_histGraphs userdata(dhistogram)=dhistlimits
	//DoWindow/F $k_histGraphs
	
	return 0
end

// show/hide histogram tab
// how = 0 show
// how = 1 hide
Function disp_ShowHideHistograms(how)
	variable how
	
	SetWindow $k_histGraphs, hide=how
	return 0
end

// append the panel
Function disp_AppendImageToolsPanel(LoR)
	variable LoR
	
	// panel frame
	DoWindow/F $k_imgDisplay
	switch(LoR)
		case 0:
			NewPanel/EXT=(LoR)/HOST=$k_imgDisplay/W=(0,0,pwidth,pheight)/N=$k_panelName/K=1 as "Image Tools Panel"
			break
		case 1:
			NewPanel/EXT=(LoR)/HOST=$k_imgDisplay/W=(pwidth,0,0,pheight)/N=$k_panelName/K=1 as "Image Tools Panel"
			break
	endswitch
#ifndef DEBUG
	ModifyPanel fixedSize=1, noEdit=1
#endif
	
	DefaultGuiFont/W=# all={"_IgorSmall",10,0}, button={"_IgorMedium",12,0}
	DefaultGuiFont/W=# tabcontrol={"_IgorMedium",14,0}, panel={"_IgorMedium",12,0}
	
	// version information
	SetDrawEnv fsize=9
	DrawText pwidth-70,15,"v: " + k_strversion
	
	// tab controls
	
	TabControl tab_main,pos={0,ptabpos},size={pwidth,pheight-ptabpos},proc=X_TabProc

	TabControl tab_main,tabLabel(0)="Load"
	//TabControl tab_main,help={"Load TIFF images."}

	TabControl tab_main,tabLabel(1)="Calibrate"
	//TabControl tab_main,help={"Calibrate xy,z."}

	TabControl tab_main,tabLabel(2)="Process"
	//TabControl tab_main,help={"Process: Threshold"}

	TabControl tab_main,tabLabel(3)="Print"
	//TabControl tab_main,help={"Process: Threshold"}
	
	disp_Tab3Draw(pwidth,pheight,ptabpos)
	disp_Tab2Draw(pwidth,pheight,ptabpos)
	disp_Tab1Draw(pwidth,pheight,ptabpos)
	disp_Tab0Draw(pwidth,pheight,ptabpos)
	disp_TabMDraw()
	
	return 0
end

// put the panel left or right
Function disp_SetImageSide(LoR)
	variable LoR

	DoWindow/K $k_imgDisplay
	disp_CreateImageWindow(LoR)
	disp_AppendImageToolsPanel(LoR)
	switch(LoR)
		case 0:		// left
			Button buttonImgL_tabM, win=$k_fullPanel, disable=2
			Button buttonImgR_tabM, win=$k_fullPanel, disable=0
			break
		case 1:	// right
			Button buttonImgL_tabM, win=$k_fullPanel, disable=0
			Button buttonImgR_tabM, win=$k_fullPanel, disable=2
			break
	endswitch

	return 0
End


// draw tabMAIN
Function disp_TabMDraw()

	variable tvoffset = 10
	variable nimgs = 0, nstacks = 1
	variable dtvoffset = 30
	
	// tab MAIN - common to all

	// Display Image Section
	
	TitleBox display_tabM, pos={35,tvoffset},size={122,22},title="Image Display"
	TitleBox display_tabM,fSize=16,frame=0,fStyle=1

	Button buttonImgL_tabM, title="|<",pos={10,10},size={20,20},proc=X_ButtonProc, disable=2
	Button buttonImgR_tabM, title=">|",pos={pwidth-25,10},size={20,20},proc=X_ButtonProc, disable=0
	
	ValDisplay valdispNimgs_tabM title="Sets Loaded",pos={200,tvoffset+2},size={120,20}, fSize=12
	ValDisplay valdispNimgs_tabM value=f_NImagesLoaded()

	// current image information
	
	tvoffset += 0.8*dtvoffset
	
	nimgs = f_NImagesLoaded()
	nstacks = f_NImagesInStack()
	
	if (nimgs > 1)
		Button buttonleft_tabM title="<",pos={20,tvoffset},size={20,20},proc=X_ButtonProc,disable=2
		Button buttonright_tabM title=">",pos={45,tvoffset},size={20,20},proc=X_ButtonProc,disable=0
	else
		Button buttonleft_tabM title="<",pos={20,tvoffset},size={20,20},proc=X_ButtonProc,disable=2
		Button buttonright_tabM title=">",pos={45,tvoffset},size={20,20},proc=X_ButtonProc,disable=2
	endif

	PopupMenu popupfolder_tabM,title="",mode=1,value= #"f_LoimgT()"
	PopupMenu popupfolder_tabM,align=0,pos={70,tvoffset},proc=X_PopMenuProc
		
	tvoffset += dtvoffset
	
	PopupMenu popupfile_tabM,pos={20,tvoffset},size={106,23},proc=X_PopMenuProc,title=""
	PopupMenu popupfile_tabM,mode=1,value= #"f_LoimgF()",disable=0
	
	SetVariable setvarSN_TabM, title="stack",pos={pwidth-255,tvoffset+2},size={80,20},proc=X_SetVarProc,fSize=10
	SetVariable setvarSN_TabM, limits={0,nimgs-1,1}, value=_NUM:0, disable=(nstacks<1)

	ValDisplay valdispNPlanes_TabM, title="/",pos={pwidth-171,tvoffset+2},size={40,20}
	ValDisplay valdispNPlanes_TabM, value=_NUM:(nimgs-1), disable=(nstacks<1)
	
	Button buttondoMovie_tabM, pos={pwidth-125,tvoffset}, size={30,20}, proc=X_ButtonProc
	Button buttondoMovie_tabM, title="🎥", disable=(nstacks<1)
	
	SetVariable setvarsleep_tabM, title="sleep",pos={pwidth-90,tvoffset+2},size={75,20},proc=X_SetVarProc
	SetVariable setvarsleep_tabM, limits={0,100,10}, value=_NUM:0, disable=(nstacks<1)

	tvoffset += dtvoffset
			
	SetVariable setvarSize_tabM,pos={20,tvoffset+2},title="XY Size",size={280,20}
	SetVariable setvarSize_tabM,value=_STR:f_ImageSizeText(),noedit=1
	ValDisplay valdispBitDepth_tabM,pos={320,tvoffset+2},title="Bit Depth",size={80,20}
	ValDisplay valdispBitDepth_tabM,value=_NUM:f_ImageBitDepth()
	
	tvoffset += 0.8*dtvoffset
	
	SetVariable setvarImgNote_tabM,pos={20,tvoffset},title="Notes",size={380,20}
	SetVariable setvarImgNote_tabM,value=_STR:f_ImageNoteText(), proc=X_SetVarProc
	
	tvoffset += dtvoffset
	
	SetDrawEnv linefgc=(30583,30583,30583),fillpat=0,dash=2
	DrawLine/W=$k_fullpanel 0,tvoffset,pwidth,tvoffset
	
	// marquee
	
	tvoffset += 0.8*dtvoffset

	TitleBox marqueeHeader_tabM,pos={15,tvoffset},size={122,22},title="Marquee"
	TitleBox marqueeHeader_tabM,fSize=16,frame=0,fStyle=1,disable=0
		
	//tvoffset += dtvoffset
	
	SetVariable mqx0_tabM, title="<",pos={120,tvoffset-8},size={70,20},value=_NUM:NaN,proc=X_SetVarProc,disable=0
	SetVariable mqy0_tabM, title="^",pos={200,tvoffset-8},size={70,20},value=_NUM:NaN,proc=X_SetVarProc,disable=0
	SetVariable mqdx_tabM, title="-",pos={120,tvoffset+15},size={70,20},value=_NUM:NaN,proc=X_SetVarProc,disable=0
	SetVariable mqdy_tabM, title="|",pos={200,tvoffset+15},size={70,20},value=_NUM:NaN,proc=X_SetVarProc,disable=0	

	// Information Control Section
	
	tvoffset += 1.5*dtvoffset
	
	SetDrawEnv linefgc= (30583,30583,30583),fillpat=0,dash=2
	DrawLine/W=$k_fullpanel 0,tvoffset,pwidth,tvoffset
	
	tvoffset += 0.5*dtvoffset

	TitleBox infodisplay_tabM,pos={15,tvoffset},size={122,22},title="Information Display"
	TitleBox infodisplay_tabM,fSize=16,frame=0,fStyle=1,disable=0

	//tvoffset -= dtvoffset
	
	// other options
	
	PopupMenu popupWhatInfo_tabM title="",pos={pwidth-160,tvoffset},fSize=14,size={20,20}
	PopupMenu popupWhatInfo_tabM value=#"f_LoInfoOptions()",mode=2,proc=X_PopMenuProc, disable=0
		
	Button buttonRedoH_tabM,pos={pwidth-60,tvoffset-3},size={40,25},proc=X_ButtonProc,title="Redo"
	Button buttonRedoH_tabM,fSize=10,disable=0
	
	tvoffset += 0.8*dtvoffset

	PopupMenu popupHistRoI_tabM title="RoI",pos={15,tvoffset},fSize=12,size={60,20}
	PopupMenu popupHistRoI_tabM value=#"f_LoHMRoI()",proc=X_PopMenuProc,disable=0

	CheckBox checklockhRoI_tabM,pos={20,tvoffset+20},size={20,20},title="lock",disable=2
	CheckBox checklockhRoI_tabM,fSize=10,value=0,proc=X_CheckProc,userdata(state)="2"

	tvoffset += dtvoffset

	ValDisplay hmqx0_tabM, title="<",pos={120,tvoffset-8},size={65,20},value=_NUM:NaN,disable=1
	ValDisplay hmqy0_tabM, title="^",pos={200,tvoffset-8},size={65,20},value=_NUM:NaN,disable=1
	ValDisplay hmqdx_tabM, title="-",pos={120,tvoffset+15},size={65,20},value=_NUM:NaN,disable=1
	ValDisplay hmqdy_tabM, title="|",pos={200,tvoffset+15},size={65,20},value=_NUM:NaN,disable=1
	
	TitleBox histscale_tabM,pos={290,tvoffset-20},size={122,22},title="dI/dB"
	TitleBox histscale_tabM,fSize=10,frame=0,disable=0
	
	Slider slidermagdhist_tabM pos={300,tvoffset-7},value=0,limits={0,3,0.2}
	Slider slidermagdhist_tabM proc=X_SliderProc,size={65,40},ticks=0,disable=0
	
	ValDisplay valdispcx_tabM pos={320,tvoffset-10}, title="x",size={60,20},value=_NUM:xcsr(A,k_histgraphs),fSize=10,disable=0
	ValDisplay valdispcy_tabM pos={320,tvoffset+10}, title="y",format="%3.3g", size={75,20},value=_NUM:vcsr(A,k_histgraphs),fSize=10,disable=0

	// Profile Values Section
//	
//	tvoffset -= 0.5*dtvoffset
////
////	Display/N=$k_mqprofile/HOST=#/W=(170,tvoffset,pwidth-10,tvoffset+75)
////	disp_AppendMQProfile()
//	
//	PopupMenu popupMQProfile_tabM,title="Profile",pos={15,tvoffset}, proc=X_PopMenuProc
//	PopUpMenu popupMQProfile_tabM mode=1,value="- Horizontal;| Vertical;",fSize=12, disable=1
//
//	tvoffset += dtvoffset
//
//	//tvoffset += dtvoffset
//		
//	ValDisplay valdispmqdx_tabM title="dx [p]",pos={140,tvoffset},size={80,20}, fSize=10, disable=1
//	//ValDisplay valdispmqdx_tabM value=_NUM:f_MQCursorDelta(0), disable=0
//
//	ValDisplay valdispmqdxm_tabM title="dx [m]",pos={235,tvoffset},size={80,20}, fSize=10, disable=1
//	//ValDisplay valdispmqdxm_tabM value=_NUM:f_MQCursorDelta(1), format="%1.3g", disable=0
//		
//	ValDisplay valdispmqdy_tabM title="dy",pos={330,tvoffset},size={80,20}, fSize=10, disable=1
//	//ValDisplay valdispmqdy_tabM value=_NUM:f_MQCursorDelta(2), disable=0

	return 0
end

// ***
// SUPPORT FUNCTIONS

// initialize menu options for a given tab
Function disp_InitializeTabMenus(tab)
	variable tab
	
	DFREF pf=$k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
#ifdef DEBUG
	print "initializing tab ", tab
#endif

	switch(tab)
		case -1:		// initialize package
			PopupMenu/Z popupfolder_tabM win=$k_fullpanel, disable = 1
			PopupMenu/Z popupfile_tabM win=$k_fullpanel, disable = 1
			Button/Z buttonleft_tabM win=$k_fullpanel, disable = 1
			Button/Z buttonright_tabM win=$k_fullpanel, disable = 1
			break
		case 1:
			break
		case 100:		// load first image
			SetImagetoShow(cwdfolder, cwfile)
			PopupMenu/Z popupfolder_tabM win=$k_fullpanel, disable = 0
			PopupMenu/Z popupfile_tabM win=$k_fullpanel, disable = 0
			disp_ResetImageFileonPanel()
			disp_UpdateTabM()
			break
	endswitch
	
	return 0
end

// lock the histogram to a RoI
Function LockHistogramRoI(yn)
	variable yn

	variable hq, ppx0, ppx1, ppy0, ppy1
	STRUCT S_MarqueeCoordinates mc

	switch(yn)
		case 0:		// unlock
			hq = Sf_GetHRoICoordinates(mc)
			if (hq==-1)
				return -1
			endif
			SetDrawLayer/W=$k_imgDisplay ProgFront
			DrawAction/W=$k_imgDisplay getgroup=hRoI, delete
			SetDrawLayer/W=$k_imgDisplay UserFront
			SetMarquee/W=$k_imgDisplay, mc.paleft,mc.patop,mc.paright,mc.pabottom
			break
		case 1:		// lock
			hq = Sf_GetMQCoordinates(mc)
			if (hq==-1)
				return -1
			endif
			// put trunated pixel coordinates in local variables
			ppx0 = trunc(mc.pgleft)
			ppy0 = trunc(mc.pgtop)
			ppx1 = trunc(mc.pgright)
			ppy1 = trunc(mc.pgbottom)
			SetDrawLayer/W=$k_imgDisplay ProgFront
			SetDrawEnv/W=$k_imgDisplay gstart,gname=hRoI
			SetDrawEnv/W=$k_imgDisplay linefgc=(65535,0,0),linethick=2,fillpat=0, xcoord=top,ycoord=left, save
			DrawRect/W=$k_imgDisplay ppx0,ppy0,ppx1,ppy1
			SetDrawEnv/W=$k_imgDisplay gstop
			SetDrawLayer/W=$k_imgDisplay UserFront
			break
	endswitch
	
	return 0
end

// split display windows
Function disp_SplitWindow(how)
	variable how
	
	DFREF pf=$k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string rfile = f_RootFileName()
	string tfile = rfile + "_th"
	string bfile = rfile + "_bg"
	string wdf = "root:" + cwdfolder
	
	switch(how)
		case 0:		// refresh entirely (with no prior information)
			rfile = ImageNameList(k_imgDisplay,";")
			if (ItemsInList(rfile) > 1)
				tfile = StringFromList(1,rfile)
				RemoveImage/W=$k_imgDisplay/Z $tfile
				ModifyGraph/W=$k_imgDisplay axisEnab(top)={0,1}
				bfile = StringFromList(0,f_SplitOptions())
				PopupMenu popupSplit_CB, win=$k_imgDisplay, mode=1, value=f_SplitOptions(), userdata=bfile
			endif
			break
		case 1:		// none
			RemoveImage/W=$k_imgDisplay/Z $tfile
			RemoveImage/W=$k_imgDisplay/Z $bfile
			ModifyGraph/W=$k_imgDisplay axisEnab(top)={0,1}
			break
		case 2:		// threshold
			wave/SDFR=$wdf timg = $tfile
			RemoveImage/W=$k_imgDisplay/Z $bfile
			AppendImage/W=$k_imgDisplay/T=splitT timg
			if (f_isStack())
				ModifyImage/W=$k_imgDisplay $tfile, plane=f_PlaneNumber()
			endif
			ModifyGraph/W=$k_imgDisplay axisEnab(top)={0,0.5},axisEnab(splitT)={0.5,1}
			ModifyGraph/W=$k_imgDisplay noLabel(splitT)=2,nticks(splitT)=0,freePos(splitT)=0
			break
		case 3:		// background
			wave/SDFR=$wdf bimg = $bfile
			RemoveImage/W=$k_imgDisplay/Z $tfile
			AppendImage/W=$k_imgDisplay/T=splitT bimg
			if (f_isStack())
				ModifyImage/W=$k_imgDisplay $bfile, plane=f_PlaneNumber()
			endif
			ModifyGraph/W=$k_imgDisplay axisEnab(top)={0,0.5},axisEnab(splitT)={0.5,1}
			ModifyGraph/W=$k_imgDisplay noLabel(splitT)=2,nticks(splitT)=0,freePos(splitT)=0
			break	
	endswitch
	
	return 0
end

// function to set Histogram RoI
Function disp_SetHRoIControls(onoff)
	variable onoff
	
	switch(onoff)
		case 0:		// off
			ValDisplay hmqx0_tabM, win=$k_fullpanel,value=_NUM:NaN
			ValDisplay hmqy0_tabM, win=$k_fullpanel,value=_NUM:NaN
			ValDisplay hmqdx_tabM, win=$k_fullpanel,value=_NUM:NaN
			ValDisplay hmqdy_tabM, win=$k_fullpanel,value=_NUM:NaN
			break
		case 1:		// on
			ControlInfo/W=$k_fullpanel mqx0_tabM
			ValDisplay hmqx0_tabM, win=$k_fullpanel,value=_NUM:v_value
			ControlInfo/W=$k_fullpanel mqy0_tabM
			ValDisplay hmqy0_tabM, win=$k_fullpanel,value=_NUM:v_value
			ControlInfo/W=$k_fullpanel mqdx_tabM
			ValDisplay hmqdx_tabM, win=$k_fullpanel,value=_NUM:v_value
			ControlInfo/W=$k_fullpanel mqdy_tabM
			ValDisplay hmqdy_tabM, win=$k_fullpanel,value=_NUM:v_value
			break
	endswitch
				
	return 0
end

// function to update TabM
Function disp_UpdateTabM()

	DFREF pf=$k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	variable iwd, iwf, istack, shinfo, ninfoopts, pld = f_PlaneNumber()
	string lowfolders = f_LoimgT()
	string lowfiles = f_LoimgF(sfolder=cwdfolder)

	// update list of folder sets and list of files in current folder
	iwd = WhichListItem(cwdfolder[5,inf],lowfolders)+1
	iwf = WhichListItem(cwfile[4,inf],lowfiles)+1
	lowfiles = "\"" + lowfiles + "\""
	
	// update folder and file popups
	PopupMenu popupfolder_tabM, win=$k_fullpanel, value=f_LoimgT(), mode=iwd, userData=cwdfolder
	PopupMenu popupfile_tabM, win=$k_fullpanel, value=#lowfiles, mode=iwf, userData=cwfile
	ValDisplay valdispNimgs_tabM, win=$k_fullpanel, value=f_NImagesLoaded()
	
	// handle cases for too few number of images loaded
	if (ItemsInList(f_LoimgT())<2)
		Button/Z buttonleft_tabM, win=$k_fullpanel, disable=2
		Button/Z buttonright_tabM,win=$k_fullpanel, disable=2
	elseif (iwd == 1)
		Button/Z buttonleft_tabM, win=$k_fullpanel, disable=2
		Button/Z buttonright_tabM,win=$k_fullpanel, disable=0
	elseif (iwd == ItemsInList(f_LoimgT()))
		Button/Z buttonleft_tabM, win=$k_fullpanel, disable=0
		Button/Z buttonright_tabM,win=$k_fullpanel, disable=2
	else
		Button/Z buttonleft_tabM, win=$k_fullpanel, disable=0
		Button/Z buttonright_tabM,win=$k_fullpanel, disable=0
	endif
	
	// update image size, note, and bit depth
	SetVariable setvarSize_tabM, win=$k_fullpanel, value=_STR:f_ImageSizeText()
	SetVariable setvarImgNote_tabM, win=$k_fullpanel, value=_STR:f_ImageNoteText()
	ValDisplay valdispBitDepth_tabM, win=$k_fullpanel, value=_NUM:f_ImageBitDepth()
	
	// update if stack or not
	istack = f_NImagesinStack()
	SetVariable setvarSN_tabM, win=$k_fullPanel, limits={0,iStack-1,1}, value=_NUM:pld, disable=(iStack==0)
	Button buttondoMovie_tabM, win=$k_fullPanel, disable=(iStack==0)
	SetVariable setvarsleep_tabM, win=$k_fullPanel, disable=(iStack==0)
	ValDisplay valdispNPlanes_TabM, win=$k_fullPanel, value=_NUM:(iStack-1), disable=(iStack==0)
		
	// update marquee
	if (!f_HasMQ())
		SetVariable mqx0_tabM, win=$k_fullpanel, value=_NUM:NaN
		SetVariable mqy0_tabM, win=$k_fullpanel, value=_NUM:NaN
		SetVariable mqdx_tabM, win=$k_fullpanel, value=_NUM:NaN
		SetVariable mqdy_tabM, win=$k_fullpanel, value=_NUM:NaN
	endif
	
	// information display
	PopupMenu popupWhatInfo_tabM, win=$k_fullpanel, value=#"f_LoInfoOptions()"
	ControlInfo/W=$k_fullPanel popupWhatInfo_tabM
	shinfo = v_value
	disp_UpdateInfo(shinfo)
	
#ifdef DEBUG
	print "updated Tab M"
#endif
	
	return 0
end

// update Info Window
Function disp_UpdateInfo(mode)
	variable mode
	
	switch(mode)
		case 1:		// none
			PopupMenu popupHistRoI_tabM, win=$k_fullPanel, disable=1
			Checkbox checklockhRoI_tabM, win=$k_fullPanel, disable=1
			ValDisplay hmqx0_tabM, win=$k_fullpanel,disable=1
			ValDisplay hmqy0_tabM, win=$k_fullpanel,disable=1
			ValDisplay hmqdx_tabM, win=$k_fullpanel,disable=1
			ValDisplay hmqdy_tabM, win=$k_fullpanel,disable=1
			TitleBox histscale_tabM,win=$k_fullPanel, disable=1
			Slider slidermagdhist_tabM, win=$k_fullPanel, disable=1
			ValDisplay valdispcx_tabM win=$k_fullPanel, disable=1
			ValDisplay valdispcy_tabM win=$k_fullPanel, disable=1
			Button buttonRedoH_tabM, win=$k_fullPanel, disable=1
			PopUpMenu popupMQProfile_tabM, win=$k_fullPanel, disable=1
			ValDisplay valdispmqdx_tabM, win=$k_fullpanel, disable=1
			ValDisplay valdispmqdxm_tabM, win=$k_fullpanel, disable=1
			ValDisplay valdispmqdy_tabM, win=$k_fullpanel, disable=1

			disp_ShowHideHistograms(1)
			break
		case 2:		// histograms
			PopupMenu popupHistRoI_tabM, win=$k_fullPanel, disable=0
			// handle case for marquee or HRoI
			if (f_HasMQ() || f_HasHRoI())
				Checkbox checklockhRoI_tabM, win=$k_fullPanel, disable=0
			else
				Checkbox checklockhRoI_tabM, win=$k_fullPanel, disable=2
			endif
			ControlInfo/W=$k_fullpanel checklockhRoI_tabM
			ValDisplay hmqx0_tabM, win=$k_fullpanel,disable=(!v_value)
			ValDisplay hmqy0_tabM, win=$k_fullpanel,disable=(!v_value)
			ValDisplay hmqdx_tabM, win=$k_fullpanel,disable=(!v_value)
			ValDisplay hmqdy_tabM, win=$k_fullpanel,disable=(!v_value)
			TitleBox histscale_tabM,win=$k_fullPanel, disable=0
			Slider slidermagdhist_tabM, win=$k_fullPanel, disable=0
			ValDisplay valdispcx_tabM win=$k_fullPanel, disable=0
			ValDisplay valdispcy_tabM win=$k_fullPanel, disable=0
			Button buttonRedoH_tabM, win=$k_fullPanel, disable=0
			PopUpMenu popupMQProfile_tabM, win=$k_fullPanel, disable=1
			ValDisplay valdispmqdx_tabM, win=$k_fullpanel, disable=1
			ValDisplay valdispmqdxm_tabM, win=$k_fullpanel, disable=1
			ValDisplay valdispmqdy_tabM, win=$k_fullpanel, disable=1
			disp_ShowHideHistograms(0)
//				
//			switch(f_HasMQ())
//				case 0:		// no marquee or RoI
//					//ValDisplay valdispcx_tabM, win=$k_fullPanel, disable=((v_value==0))
//					//ValDisplay valdispcy_tabM, win=$k_fullPanel, disable=((v_value==0))
//					break
//				case 1:		// image marquee
//						Checkbox checklockhRoI_tabM, win=$k_fullPanel, disable=0
//					break
//			endswitch
//			if (!f_HasMQ())
//				GenerateMQProfile(0)
//			else
//				ControlInfo/W=$k_fullpanel popupMQprofile_tabM
//				GenerateMQProfile(v_value)
//			endif
//			disp_UpdateMQProfileCsrValues()
			break		
		case 3:		// profiles
			break
	endswitch
	
	return 0
	
end

// update panel control displays and values for a given tab
Function disp_UpdatePanelControls(tab)
	variable tab
	
#ifdef DEBUG
	print "updating tab ", tab
#endif

	switch(tab)
		case -1:		// main (always present) content
			disp_updateTabM()
			break
		case 0:		// load tab
			disp_updateTab0()
			break
		case 1:		// calibrate tab
			disp_updateTab1()
			break
		case 2:		// process tab
			disp_updateTab2()
			break
		case 3:		// print tab
			disp_updateTab3()
			break
	endswitch
	
	return 0
end

// ***
// SHOW IMAGE

// show an image
Function disp_ShowAnImage([plane,reset])
	variable plane,reset

	// globals for package	
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string fstr, tstr
	variable bd
	
	// just moving to new plane
	if (!ParamIsDefault(plane) && (ParamIsDefault(reset)))
		ModifyImage/W=$k_imgDisplay $cwfile, plane=plane
		fstr = f_SplitWindowImage()
		if (strlen(fstr)!=0)
				ModifyImage/W=$k_imgDisplay $fstr, plane=plane
		endif
		return 0
	endif

#ifdef DEBUG
	print "Showing an image ", cwfile
#endif
	
	// set the folder and new image to display
	fstr = "root:" + cwdfolder
	DFREF imgF = $fstr	
	wave/SDFR=imgF itd=$cwfile
	bd = f_ImageBitDepth()
	
	tstr = StringFromList(0,(ImageNameList(k_imgDisplay,";")))
	disp_ShowScaleBar(0)
	ReplaceWave/W=$k_imgDisplay image=$tstr, itd
	fstr += ":" + cwfile
	if (f_isStack())
		ModifyImage/W=$k_imgDisplay $cwfile, plane=f_PlaneNumber()
	endif
	return 0
end

// step thru images
// -1 back; 0 stay; 1 forward
Function TM_StepThruImages(how,[plane])
	variable how, plane
	
	string simg = f_LoimgT()
	variable nimg = ItemsInList(simg), itd, nplane

#ifdef DEBUG
	print "Stepping through images ", how
#endif
	
	ControlInfo/W=$k_fullpanel popupFolder_tabM
	itd = v_value

	if (ParamIsDefault(plane))
		switch(how)
			case -1:		// step back one image
				if (itd == 1)
					return 0
				endif
				ResetImageFolder(itd-1)
				break
			case 0:		// new image
				ResetImageFolder(itd)
				break
			case 1:		// step forward one image
				if (itd == nimg)
					return 0
				endif
				ResetImageFolder(itd+1)
				break
		endswitch
		// have we switched to an image stack?
		nimg = f_NImagesInStack()
		if (nimg > 0)
			ControlInfo/W=$k_fullpanel setvarSN_TabM
			if (v_value > nimg-1)
				SetVariable setvarSN_TabM, win=$k_fullpanel, value=_NUM:(nimg-1)
				ControlUpdate/W=$k_fullpanel setvarSN_TabM
				disp_ShowAnImage(plane=(nimg-1),reset=1)
			else
				disp_ShowAnImage(plane=v_value,reset=1)
			endif
		else
			disp_ShowAnImage()
		endif
	else
		disp_ShowAnImage(plane=plane)
	endif
	
	// confirm that the marquee is still within bounds
	if (f_HasMQ())
		ResetMQCoordinates(force=1)
	endif
	
	// show histogram only when window is present
	GetWindow $k_histgraphs hide
	if (v_value==0)
		disp_ShowHistograms(1)
	endif

#ifdef DEBUG
	print "Done stepping through images ", how
#endif
	
	return 0
end

// play movie for stack
Function TM_PlayStack(slt)
	variable slt

	variable ic, ni, cplane

	if (!f_isStack())
		return 0
	endif
	
	ni = f_NImagesinStack()
	cplane = f_PlaneNumber()

#ifdef DEBUG
	print "playing a movie: ", slt
#endif
//	DoWindow/F $k_imgDisplay
//	DoUpdate
	
	try
		if (slt == 0)
			for (ic=0;ic<ni;ic+=1)
				disp_ShowAnImage(plane=ic)
				DoUpdate/W=$k_imgDisplay
				SetVariable setvarSN_TabM, win=$k_fullPanel, value=_NUM:ic
			endfor
		else
			for (ic=0;ic<ni;ic+=1)
				disp_ShowAnImage(plane=ic)
				DoUpdate/W=$k_imgDisplay
				SetVariable setvarSN_TabM, win=$k_fullPanel, value=_NUM:ic
				Sleep/T/Q slt
			endfor
		endif
		disp_ShowAnImage(plane=cplane)
		DoUpdate/W=$k_imgDisplay
		SetVariable setvarSN_TabM, win=$k_fullPanel, value=_NUM:cplane
	catch
		SetVariable setvarSN_TabM, win=$k_fullPanel, value=_NUM:(ic-1)
		TM_StepThruImages(0,plane=(ic-1))
	endtry
	
	// return focus to panel after running movie
	DoWindow/F $k_panelName
	return 0
end

// reset marquee coordinates
Function ResetMQCoordinates([force])
	variable force

	variable left, top, right, bottom
	variable gleft, gtop, gright, gbottom
	variable aleft, awidth, atop, aheight
	variable pxlsh, pntsh, pntsppxl, ex
	
	if (!ParamIsDefault(force))
		GetMarquee/W=$k_imgDisplay left, top
		SetVariable mqx0_tabM, win=$k_fullpanel, disable=0, value=_NUM:round(v_left)
		SetVariable mqdx_tabM, win=$k_fullpanel, disable=0, value=_NUM:ceil(v_right - v_left)
		SetVariable mqy0_tabM, win=$k_fullpanel, disable=0, value=_NUM:round(v_top)
		SetVariable mqdy_tabM, win=$k_fullpanel, disable=0, value=_NUM:ceil(v_bottom - v_top)
	endif
	
	GetWindow $k_imgDisplay, psizeDC
	pxlsh = V_right - V_left
	GetWindow $k_imgDisplay, psize
	pntsh = V_right - V_left
	pntsppxl = pntsh / pxlsh
	
	GetWindow $k_imgDisplay expand
	ex = v_value == 0 ? 1 : v_value
	GetAxis/W=$k_imgDisplay/Q top
	gleft = V_min
	gright = V_max
	GetAxis/W=$k_imgDisplay/Q left
	gtop = V_Max
	gbottom = V_Min
	
	ControlInfo/W=$k_panelName mqx0_tabM
	aleft = v_value
	if (aleft < gleft)
		aleft = gleft
		SetVariable mqx0_tabM, win=$k_panelName, value=_NUM:0
	endif
	left = ex*(PixelFromAxisVal(k_imgDisplay,"left",aleft)*pntsppxl)
	ControlInfo/W=$k_panelName mqy0_tabM
	atop = v_value
	if (atop < gtop)
		atop = gtop
		SetVariable mqy0_tabM, win=$k_panelName, value=_NUM:0
	endif
	top = ex*(PixelFromAxisVal(k_imgDisplay,"top",atop)*pntsppxl) + cbheight
	ControlInfo/W=$k_panelName mqdx_tabM
	awidth = v_value
	if ((awidth+aleft) > f_ImageSize()[1])
		awidth = f_ImageSize()[3] - aleft
		SetVariable mqdx_tabM, win=$k_panelName, value=_NUM:awidth
	endif		
	right = ex*(PixelFromAxisVal(k_imgDisplay,"left",aleft + awidth)*pntsppxl)
	ControlInfo/W=$k_panelName mqdy_tabM
	aheight = v_value
	if ((aheight+atop) > f_ImageSize()[2])
		aheight = f_ImageSize()[4] - atop
		SetVariable mqdy_tabM, win=$k_panelName, value=_NUM:aheight
	endif		
	bottom = ex*(PixelFromAxisVal(k_imgDisplay,"top",atop + aheight)*pntsppxl) + cbheight

	SetMarquee/W=$k_imgDisplay round(left), round(top), round(right), round(bottom)
	
	if (!ParamIsDefault(force))
		GetMarquee/W=$k_imgDisplay left, top
		SetVariable mqx0_tabM, win=$k_fullpanel, disable=0, value=_NUM:round(left)
		SetVariable mqdx_tabM, win=$k_fullpanel, disable=0, value=_NUM:ceil(right - left)
		SetVariable mqy0_tabM, win=$k_fullpanel, disable=0, value=_NUM:round(top)
		SetVariable mqdy_tabM, win=$k_fullpanel, disable=0, value=_NUM:ceil(bottom - top)
	endif
	
	return 0
end

// reset image folder (increment value)
Function ResetImageFolder(fnum)
	variable fnum
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	// reset the global folder
	string flist = f_LoimgT()
	
	flist = f_LoimgT()
	cwdfolder = "imgT_" + StringFromList(fnum-1,flist,";")
	
	// reset the file to the first one in the folder when needed
	flist = f_LoimgF(sfolder=cwdfolder)
	fnum = WhichListItem(cwfile[4,inf],flist)
	if (fnum < 0)
		cwfile = "img_" + StringFromList(0,f_LoimgF(sfolder=cwdfolder),";")
	else
		cwfile = "img_" + StringFromList(fnum,flist)
	endif
	
#ifdef DEBUG
	print "reset image folder/file ", cwdfolder, cwfile
#endif

	return 0
end

Function ResetImageFile(fname)
	string fname
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwfile

	cwfile = "img_" + fname
	return 0
end	
	

// reset image file on panel
Function disp_ResetImageFileonPanel()
	
	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	variable fnum
	string flist = f_LoimgF(), ffile

	// reset the file
	ffile = cwfile[4,inf]
	fnum = WhichListItem(ffile,flist)
	PopupMenu popupfile_tabM, win=$k_fullpanel, mode=fnum+1, popvalue=ffile
	SetVariable setvarImgNote_tabM, win=$k_fullpanel, value=_STR:f_ImageNoteText()
	SetVariable setvarSize_tabM,win=$k_fullpanel, value=_STR:f_ImageSizeText()
	return 0
end

// ***
// PROCESSING

// ***
// HISTOGRAMS

// generate the histogram
// toRoI - 1 means generate them within a defined RoI
Function GenerateHistograms(toRoI)
	variable toRoI
	
	DFREF cdf = GetDataFolderDFR()

	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string dfstr, imgHstr, imgroot, maskWave, sfile
	variable quiet=0, maskordraw, wtype
	variable bflag=0, tflag=0, dhmin, dhmax
	variable isstack=0, stnum, cplane
	
	dfstr = "root:" + cwdfolder
	DFREF wf = $dfstr
	
	imgroot = StringFromList(1,cwfile,"_")
	
	SetDataFolder wf
	
	// do not re-generate histogram for threshold images
	
	if (f_isThreshold())
		sfile = ReplaceString("_th",cwfile,"")
	else
		sfile = cwfile
	endif

	wave wimg = $sfile
	
	// extract only the current stack plane
	if (f_isStack()==1)
		cplane = f_PlaneNumber()
		MatrixOp/O himg = wimg[][][cplane]
		isstack = 1
	else
		duplicate/O wimg himg
	endif

#ifdef DEBUG
	print "generating histogram for ", cwdfolder, cwfile, cplane
#endif
	
	NewDataFolder/O/S $k_histFolder
	
	// 8bit or 16bit
	if (f_ImageBitDepth() == 16)
		bflag = 1		// 16 bit
	endif
	
	// single channel or three channel
	tflag = f_imgRGB(thisimg=himg)
	
	// check whether mq or drawing exists for histogram
	if (toRoI > 0)
		if (!(f_HasMQ() || f_HasHRoI()))
			toRoI = 0
		endif
	endif
	
	// get marquee region
	STRUCT S_MarqueeCoordinates mc
	
	switch(toRoI)
		case 0:	// all image
			break
		default:		
			if (f_HasHRoI()==1)
				Sf_GetHRoICoordinates(mc)
			elseif (f_HasMQ()==1)
				Sf_GetMQCoordinates(mc)
			endif
			break
	endswitch
	
	// generate histogram	
	switch(toRoI)
		case 0:		// histogram over all image
			if (bflag)
				ImageHistogram/I himg
			else
				ImageHistogram himg
			endif
			if (!tflag)
				wave W_imageHist
			else
				wave W_imageHistR, W_imageHistG, W_imageHistB
			endif
			break
		case 3:		// outside - inside
		case 4:		// inside - outside
			if (tflag)
				break
			endif
			toRoI = toRoI == 3 ? -1 : 1
			// get outside
			//maskWave = "himg_HRoIMask"
			//duplicate/O himg $maskWave
			wave RoIMask = GenerateRoIMask(himg,mc,1,0)
			if (bflag)
				ImageHistogram/I/R=RoIMask himg
			else
				ImageHistogram/R=RoIMask himg
			endif
			wave W_imageHist
			duplicate/FREE W_imageHist iHoutside
			// get inside
			wave RoIWave = GenerateRoIMask(himg,mc,2,0)
			if (bflag)
				ImageHistogram/I/R=RoIMask himg
			else
				ImageHistogram/R=RoIMask himg
			endif
			wave W_imageHist
			W_imageHist -= iHoutside
			W_imageHist *= toRoI
			W_imageHist = W_imageHist < 0 ? 0 : W_imageHist
			//W_imageHist = abs(W_imageHist)
			break
		// case 0 - all, 1 - outside, 2 - inside
		default:		// histogram with RoI
			if (tflag)
				break
			endif
			//maskWave = "himg_HRoIMask"
			//duplicate/O himg $maskWave
			wave RoIMask = GenerateRoIMask(himg,mc,toRoI,0)
			if (bflag)
				ImageHistogram/I/R=RoIMask himg
			else
				ImageHistogram/R=RoIMask himg
			endif
			wave W_imageHist
			break
	endswitch
	
	switch(tflag)
		case 0:		// single channel image
			wave iHo = W_imageHist
			break
		case 1:		// rgb image
			wave iHRo = W_imageHistR
			wave iHGo = W_imageHistG
			wave iHBo = W_imageHistB
			duplicate/O/FREE iHRo, iHo
			iHo += iHGo + iHBo
			break
	endswitch	
	
	imgHstr = "hist_" + imgroot	
	duplicate/O iHo, $imgHstr
	wave iH = $imgHstr
	// 16 bit rescale to step of 1
	if (bflag==1)
		SetScale/P x 0,1,"", iH
	endif
	
	imgHstr = "chist_" + imgroot
	duplicate/O iH, $imgHstr
	wave iCH = $imgHstr
	integrate/T iCH /D=iCH
	
	imgHstr = "dhist_" + imgroot
	duplicate/O iH, $imgHstr
	wave iDH = $imgHstr
	differentiate iDH /D=iDH
	WaveStats/Q iDH
	dhmax = max(abs(v_max),abs(v_min))
	iDH /= dhmax
	
#ifndef DEBUG
	killwaves/Z RoIMask, himg, W_imageHist, W_imageHistR, W_imageHistG, W_imageHistB
#endif
//
//	if (isstack)
//		duplicate/O sstack_tmp wimg
//	endif
	
	SetDataFolder cdf

	return 0
end

// generate an RoI mask from wimg
// how: 1 mask inside; 2 mask outside
// which: 0 histogram, 1 background, 2 threshold
Function/WAVE GenerateRoIMask(wimg,mc,how,which,[thextra])
	wave wimg
	STRUCT S_MarqueeCoordinates &mc
	variable how, which, thextra

	variable rtn = 0, roi, mask, npts, ic
	variable ppx0, ppx1, ppy0, ppy1
	string wname = NameOfWave(wimg)
	
	// set for background
	roi = 64
	mask = 0
	
	switch(which)
		case 0:	// histogram
			roi = 0
			mask = 64
		case 3:	// threshold
			// put trunated pixel coordinates in local variables
			ppx0 = ScaleToIndex(wimg,mc.pgleft,0)
			ppx1 = ScaleToIndex(wimg,mc.pgright,0)
			ppy0 = ScaleToIndex(wimg,mc.pgtop,1)
			ppy1 = ScaleToIndex(wimg,mc.pgbottom,1)

			// store RoI values for the displayed image
			duplicate/FREE wimg mwimg
			wave imgMask = mwimg
			if (WaveDims(wimg) == 3)
				Redimension/B/U/N=(-1,-1,1) imgMask
			else
				Redimension/B/U imgMask
			endif
			// generate RoI ... all image of interest
			switch(how)
				case 1:		// Mask inside
					imgMask = roi
					imgMask[ppx0,ppx1][ppy0,ppy1] = mask
					break
				case 2:		// Mask outside
					imgMask = mask
					imgMask[ppx0,ppx1][ppy0,ppy1] = roi
					break
			endswitch
			break
	endswitch	
	
	return mwimg
end

// scale the derivative of the histogram
Function ScaleDHistogram(sf)
	variable sf
	
	variable dhmin, dhmax
	dhmin = -(10^(-sf))
	dhmax = (10^(-sf))
	SetAxis/W=$k_histgraphs dhist dhmin,dhmax

	return 0	
end

// ***
// GENERAL PURPOSE

// set image note
Function TM_SetImageNote()

	// global values
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string rtstr = ""
	string wname = "root:" + cwdfolder + ":" + cwfile

	ControlInfo/W=$k_fullpanel setvarImgNote_tabM
	rtstr = s_value
	Note/K $wname
	Note $wname, rtstr
	
	// update contact print with new note
	ControlInfo/W=$k_fullpanel checkCP_tab3
	if (v_value)
		T3_StoreasContactPrint()
	endif

	return 0
end
